#!/usr/bin/env python2
#
# install_live.py - format partition and install live system
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This file is part of the larch project.
#
#    larch is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    larch is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with larch; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#----------------------------------------------------------------------------
# 2010.11.07

import threading, os, sys
import backend, fstab
from glob import glob
from partitions import Partitions, LIVESYS

exec backend.readdata('larchin.conf')
if os.path.isfile('/etc/larchin.conf'):
    execfile('/etc/larchin.conf')

class Installer:
    """This class manages the copying of the live system to the disk.
    It also handles peripheral requirements such as partition formatting
    and mounting, and the generation of an initramfs and initial /etc/fstab,
    bootloader installation, etc., using external modules for some of the
    steps.
    """
    def __init__(self):
        pass


        if options.livesize == 0:
            self.livelabel = None
        else:
            self.livelabel = options.livelabel if options.livelabel else livelabel


# The default is to create a partition with the given (or default) name, the
# size being incremented from the actual live system size.
# If the partition exists already, check that it meets the minimum size requirement.
# It should also be possible to pass a device path, in which case check that
# no other partition already has the desired label.


    def run(self):
        """This function is the main controller for the sequence of processes.
        """
        partitions = Partitions(dev, MP)
        ### Do partitioning
        if options.autopart:
            errormessage = partitions.autolive(self.livelabel,
                    options.livesize, options.destination)
            if errormessage:
                return errormessage

        ### Do formatting
        if options.format:
            errormessage = partitions.format(self.livelabel)
            if errormessage:
                return errormessage

        ### Mount the system
        if options.mount:
            errormessage = partitions.mount(self.livelabel, MP)
            if errormessage:
                return errormessage

        ### Perform the actual installation
        if options.do_copy and not scripts.run('copy-live', MP, LIVESYS):
            return _("Copying live system to partition '' failed") % self.livelabel

        ### extlinux
        if options.extlinux or options.do_copy:
#TODO:
# Probably need to change the label.
# If it's not booting by label, need to change it to do so.
# Would it be better to borrow the relevant stuff from larch?

#?                writefile(elconf, MP + '/boot/extlinux/extlinux.conf')
                ok, res = run_capture('extlinux', '--install',
                        MP + '/boot/isolinux')
                if not ok:
                    return res


#?
        ### Master Boot Record
        if options.mbr:
#TODO:
            pass



if __name__ == "__main__":
    from optparse import OptionParser, OptionGroup
    parser = OptionParser(usage=_("usage: %prog [options] [device-path (default: /dev/sda)]"))

    parser.add_option("-a", "--autopart", action="store_true",
            default="False", dest="autopart",
            help=_("Allocate a partition for the installation automatically"))
    parser.add_option("-f", "--format", action="store_true",
            default="False", dest="format",
            help=_("Format the partition"))
    parser.add_option("-i", "--install", action="store_true",
            default="False", dest="install",
            help=_("Install the live system to the destination partition"))
    parser.add_option('-d', '--destination', action='store', type='string',
            default='', dest='destination',
            help=_("Install the live system to this partition (device path)"))

    parser.add_option("-b", "--only-extlinux", action="store_true",
            default="False", dest="extlinux",
            help=_("Only install the extlinux bootloader"))

#???
    parser.add_option("-r", "--mbr", action="store_true",
            default="False", dest="mbr",
            help=_("Install a Master Boot Record to the drive and mark"
            " the installation partition as bootable"))


    parser.add_option('-L', '--live_label', action='store', type='string',
            default='', dest='livelabel',
            help=_("Label for the live partition (default: %s)") % livelabel)
    parser.add_option('-l', '--live_size', action='store', type='int',
            default='-1', dest='livesize',
            help=_("Size (MB - int) for the live partition; 0 => no live partition"))

    parser.add_option("-c", "--slave", action="store_true", dest="slave",
            default=False, help=_("Run as a slave from a controlling program"
                    " (e.g. from a gui)"))
    parser.add_option("-q", "--quiet", action="store_true", dest="quiet",
            default=False, help=_("Suppress output messages, except errors"))

    backend.init(options)

    if len(sys.argv) != 0:
        if options.destination or (len(sys.argv) > 1):
            errout(_("Unexpected argument(s): %s") % ', '.join(sys.argv), 1)
        dev = sys.argv[0]
    else:
        dev = '/dev/sda'

    # Add individual step flags
    options.mount = False
    options.do_copy = True
    if options.extlinux:
        # This is a 'single-step' option for bootloader installation,
        # no other operations will be performed.
        options.format = False
        options.autopart = False
        options.mount = True
        options.do_copy = False

    if options.installation_path:
        # If a path is supplied no partitioning or formatting should be done,
        # the destination is mounted and ready.
        options.format = False
        options.autopart = False
        options.install = False
        options.mount = False
        MP = options.installation_path
    elif options.install:
        options.mount = True
    else:
        options.do_copy = False

    if options.autopart:
        options.format = False

    installer = Installer()

    emessage = installer.run()
    if emessage:
        errout(emessage, 1)

