#!/usr/bin/env python2
#
# controller.py - Manages file-system access and calling of larchin scripts
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This file is part of the larch project.
#
#    larch is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    larch is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with larch; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#----------------------------------------------------------------------------
#2010.10.20

import sys, os, __builtin__, threading, re
from liblarch.larcon_base import Backend
from liblarch.translation import i18n_module, lang

"""
import threading
from subprocess import call
try:
    import json as serialize
except:
    import simplejson as serialize

from config import *
"""



def dummy(*args):
    return "Dummy"

exports = {'get_devices': dummy}
def add_exports(instance):
    """A simple mechanism to add fss functions. All the methods of the
    class instance which start with 'fss_' will be added to the available
    dictionary, so they can be called by fss('function-name', *args)
    [where 'function-name' is the function name without the 'fss_' prefix].
    All such functions should probably return json compatible results, to
    keep communication simple.
    """
    for attr in dir(instance):
        if attr.startswith('fss_'):
            exports[attr[4:]] = getattr(instance, attr)
__builtin__.add_exports = add_exports


def error0(message):
    sys.stderr.write('>>ERROR>>' + message + '\n')
    sys.stderr.flush()
__builtin__.error0 = error0



#TODO: the rest needs writing/modifying for larchin ...


class Fs(Backend):
    """Collect file system access methods in one class.
    """
    def __init__(self):
        Backend.__init__(self, 'larchin', globals())
        __builtin__._ = i18n_module(self.basedir, 'larchin')
        __builtin__.lang = lang
        self.event = threading.Event()

    def readfile(self, path):
        with open(path) as fh:
            return fh.read()

#    def fss_rootinit(self):
#        self.rootcall('true', self._ok)
#        return None

    def _ok(self, *args):
        self.ui_signal('done')

#    def fss_read(self, fpath, cbsig):
#        self._cbsig = cbsig
#        self.rootcall(self.basedir + '/scripts/readfile ' + fpath, self._cbdone)
#
#    def _cbdone(self, *args):
#        self.ui_signal(self._cbsig, *args)


    def fss_readdiskinfo(self):
        self.drives = []
        self.eparts = []
        for line in self.readfile('/proc/partitions').splitlines():
            m = re.search(ur'[0-9] (sd[a-z])$', line)
            if m:
                self.drives.append(m.group(1))
            m = re.search(ur'.* 1 (sd[a-z][0-9]+)', line)
            if m:
                self.eparts.append(m.group(1))

        # Get a list of mounted partitions
        self.mountedpartitions = []
        for line in self.readfile('/proc/mounts').splitlines():
            if line.startswith('/dev/sd'):
                self.mountedpartitions.append(line.split(None, 1)[0][5:])

        # Get details of each drive (run in separate thread)
        self.devinfo = []
        self.thread = threading.Thread(target=self._readinfo2_, args=())
        self.thread.start()

    def _readinfo2_(self):
        for self.dev in self.drives:
            self.partinfo = list()
            self.event.clear()
            self.rootcall('parted -m /dev/%s unit GB print free' % self.dev,
                    self._readinfo3_)
            self.event.wait()

        self.ui_signal('done', self.devinfo)
        return None

#TODO: Try to remember what the flag was for ...
    def _readinfo3_(self, flag, ok, rdpart):
        if not ok:
#TODO
            debug("Running 'parted print' on /dev/%s failed" % self.dev)
        else:
            for line in rdpart.splitlines():
                line = line.rstrip(';')
                if line[0] in '123456789':
                    info = line.split(':')
                    partsize = info[3]
                    partstart = info[1]
                    partend = info[2]
                    if info[-1] == 'free':
                        partnum = '-'
                        partfs = _("Free Space")
                        flags = ''
                    else:
                        partnum = info[0]
                        if (self.dev + partnum) in self.eparts:
                            continue
#This should probably rather be got by blkid, parted seems a little inaccurate
                        partfs = info[4]

                        flags = info[6]
                    i = len(self.partinfo)
                    while i > 0:
                        if self.pval(partstart) >= self.pval(self.partinfo[i-1][5]):
                            break
                        i -= 1
                    self.partinfo.insert(i, (partnum, partsize,
                            (_("Mounted")
                                    if (self.dev + partnum) in self.mountedpartitions
                                    else ""),
                            partfs, flags, partstart, partend))

                if line.startswith('/dev/'):
                    # Is drive removable?
                    removable = (_("Removable")
                            if self.readfile('/sys/block/%s/removable'
                                    % self.dev).startswith('1')
                            else "")
                    info = line.split(':')
                    # (device (without '/dev/'), size, removable, model, partition-list,
                    #   disk-label (msdos/gpt), sector size (logical/physical))
                    self.devinfo.append((self.dev, info[1], removable,
                            '%s (%s)' % (info[6], info[2]), self.partinfo,
                            info[5], '%s/%s' % (info[3], info[4])))

        self.event.set()
        return None


    def pval(self, textval):
        return float(textval.replace(',', '.').rstrip('GB'))



