# stage_choose_partitions.py - allocate partitions to mount-points
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This file is part of the larch project.
#
#    larch is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    larch is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with larch; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#----------------------------------------------------------------------------
# 2010.08.24

from stage import Stage
#import re

"""
This can be similarly structured to the disks page, but allows some editing.
It should be able to take over the data from the disks stage.
It allows mount-ponts to be selected for as many partitions as you like.
Formatting can also be specified (default ext4?).
Some mount options should be available (noatime, etc., auto, ...?),
maybe the default could be noatime and auto (but noauto for mounts in /mnt).
Formatting options? Only if really necessary - advanced users can format
externally.
[Possibly provide expert options, e.g. by manual input to a lineedit]
non-linux (how do I judge this with parted????), raid and lvm partitions
should not be offered.
+++
I suppose raid and lvm should be supported, if they are already set up.
That would require appropriate scans. I guess raid devices should be
treated as single partitions (unless they are lvm), even though it seems
sort of possible to put partition tables on them.
---
It might be useful to allow allocation of partition labels (maybe defaulting
to the mountpoint?).

********
The 'Go' button should probably present a page which just lists the partitions
to be used, including what is to be done with them. This page would have
'Back' and 'Install' buttons, the latter would switch to the Running View page.


****
Remember to add the adjtime stuff to the installer backend (there should be
no such relics of the live system in the final installation).
"""



class ChoosePartitions(Stage):
    def __init__(self):
        Stage.__init__(self)
        self.addwidgets('stage_choose_partitions.uim',
                (
#                    (':build*clicked', self.build),
#                    (':ssh*toggled', self.sshtoggle),
                    ('disks:dlist*changed', self.select_device),

# This is just for testing!!!:
                    ('disks:manual*clicked', self.readinfo),
               )
            )


#        self.userheaders = self.data('uheaders')


    def data(self, key):
        return self.uix('disks_page_data.get', key)


    def enter(self):
        """This is called when the page is entered/selected/shown.
        It performs initializations which depend on the state.
        """
#        docviewer.gohome('gui_disks.html')
        pass

#TODO:
# The unbusying needs to be delayed until the final part of the sequence completes
    def readinfo(self):
        """Get information on the attached hard drives (and similar devices).
        At this stage only those with device node /dev/sdX are considered.
        This runs as follows:
          Get lists of drives/devices and 'uninteresting' (essentially,
          'extended') partitions. Then read in the partition information,
          passing a list of drive info nodes to self._showinfo. Each of these
          has the form
                (   device, without '/dev/' (e.g. 'sdb'),
                    size (e.g. '2,01GB' - yes, localized!),
                    removable ("Removable" - potentially translated - or "")
                    model (e.g. 'TOSHIBA TransMemory (scsi)'),
                    list of info nodes for each partition (see below),
                    partition table type (probably 'msdos', maybe 'gpt'),
                    partition size (bytes, logical/physical, e.g. '512/512')
                )
          The partition info nodes have the form:
                (   partition index, starting at '1'),
                    size (e.g. '4,00GB'),
                    mounted ("Mounted" - potentially translated - or ""),
                    file-system (e.g. 'ext2'),
                    flags (e.g. 'boot', 'raid', 'lvm'),
                    start point (e.g. '80,0GB'),
                    end point (e.g. '84,0GB')
                )
        """
        self.fss(('readdiskinfo', self._showinfo))

    def _showinfo(self, devinfo):
        self.drivelist = devinfo
        self.currentdriveindex = None
        self.uix('disks:dlist.set', ['/dev/%s -- %8s -- %s %s' % d[:4] for d in devinfo])
        if self.currentdriveindex == None:
            self.select_device(0)


    def select_device(self, index=-1):
        if index < 0:
            return
        self.currentdriveindex = index
        d = self.drivelist[index]
        self.partitionlist = d[4]
        self.uix('disks:ptabletype.text', d[5])
        self.uix('disks:sectorsize.text', d[6])
        self.uix('disks:pinfotable.set', [p[:5] for p in self.partitionlist])






    def select_page(self, init):
        if self.run0:
            self.run0 = False
            # Test for a graphical partition manager
            if backend.available("gparted"):
                self.gparted = "gparted"
            elif backend.available("partitionmanager"):
                self.gparted = "partitionmanager"
            else:
                self.gparted = None
            self.buildgui()

        command.pageswitch(self.page_index,
                _("Disk information and selection"))






"""
TABLE
+ device-name (partition name /dev/sdb2, etc.)
+ size (GB? or as delivered by parted?)
+ mount-point
+ fs-type
+ format (checkbox)
+ ???format options???
+ ???mount-options??? (at least noatime/relatime/nodiratime/atime is useful)

If possible the partitions should be displayed in the order in which they
occupy the disk, even if this is not what parted returns or whatever.
It makes the possiblility of combining spaces etc clear even without
showing the sector boundaries.

It might help to have a separate list just showing the present situation,
non-editable (except perhaps to allow marking of partitions usable by the
automatic partitioner).

Automatic partitioning might insist on a single contiguous area, or it could
be clever enough to pick and choose (or even use LVM etc? ...). Let this
be open to ongoing development.

The table for actually choosing the mount-points should at least have the
capability of checking for LVM and RAID - perhaps via checkbox (I think
at least RAID is not so easy - it might need a bit of help?)

Can parted pick up LVM/RAID partitions? Yes ('lvm' or 'raid' in flags column)
It would be nice to be able to show at least the following types at the
first level:
    fs-type of a normal partition: ext2/3/4, ntfs/vfat, reiserfs, btrfs, etc.
    free/unused (normal partition)
    free space (unpartitioned)?
    swap
    LVM
    RAID
    extended (do I need to show this?)
    unusable (e.g. free space not in or able to be added to extended)
For actual allocation of the partitions, it would be necessary to add those
contained with the RAID/LVM areas (optionally?).

Also, if LVM/RAID is used, it might well be necessary to tweak the initcpio
configuration.

Useful - a nice touch - in the case of Arch might be the option to install
a 'pacman-cage' (a small loop-mounted fs-file to contain the pacman database,
which can help speed pacman up).

It might be simplest, at least initially, to require that the free space for
autopartitioning be at the end of the disk. I could specify the starting point
(either 'in the free space at the end', 'starting with partition X' or
'whole device'). More sophisticated approaches would be with lvm combining
marked areas, etc., but keep that for a rainy day.

The first page would then just supply disk information (maybe with a rescan button?)
and offer to start a partitioning tool (gparted or partitionmanager,
automatically choosing which one) or to autopartition the selected device.
The partitioning tool would return to the same page, after a rescan,
the autopartitioner would have a page of its own, showing its suggestion
and offering a few choices (such as the choice of usable area mentioned
above). Maybe it could use basically the same partition widget as the
manual chooser, but with its own initializations?

There is the question of when the disk should be modified. If an external
partitioner is used, this will happen immediately, but two distinct approaches
to the allocation are conceivable:
    * The existing partitions will be selected to format and have mount-points
      allocated to them.
    * The mount-points will be specified, along with sizes and file-system-types,
      how they are stored on disk is handled automatically, given the
      disk area to receive them.

This highlights the fact that there really are two separate considerations here,
which partitions do I want and where should they be stored. Because of the nature of
disk storage they are not completely independent, though using LVM or btrfs
should increase the independence considerably.
The autopartitioner should use the content angle, perhaps with some flexibility, e.g.
    + size of swap (0=> no swap)
    + size of root
    + size of home (0=> no separate /home partition)
    (sizes in GB or as % of available space: allocate first the GB, then the % as
    fractions of the remaining space. In all cases an appropriate minimum might be
    a good idea, at least colour inadequate allocations red, or some such)
    + pacman cage (0=> none)
    + optionally other partitions?
    (as an alternative to 0=>off it might be clearer/easier to use a checkbox, disabling
    the size chooser)
It could show the partitions it would remove from the existing list and there
would also be the table of the additions (with the editing options above).





USERINFO = ['pw', 'maingroup', 'uid', 'skel', 'xgroups', 'expert']

        self.userheaders = self.data('uheaders')



    def uedit(self, row, column):
        if self.usersel == row:
            uname = self.userlist[row][0]
            ulcell = self.userlist[row][column]
            if column == 4:
                ok, text = self.select_skel(ulcell)
            else:
                ok, text = ui.command('textLineDialog',
                        self.userheaders[column] + ':', 'larchify', ulcell)
            text = text.strip()
            if ok:
                try:
                    if (column == 0) and (text != ''):
                        # Rename the user, by adding a new one and deleting
                        # the old
                        uname = text
                        fss('newuser', uname)
                        i = 0
                        for f in USERINFO:
                            i += 1
                            fss('userset', uname, f, self.userlist[row][i])
                        if not fss('deluser', ulcell):
                            run_error(self.data('rn_error'))

                    else:
                        fss('userset', uname, USERINFO[column-1], text)
                        fss('saveusers')

                except:
                    run_error(self.data('ud_error'))
                self.readuserinfo(uname)

        else:
            self.usersel = row

"""








#from backend import DiskInfo

"""
<h2>Disk Selection</h2>
<p>On the basis of the detected disk(-like) devices a choice of devices
for automatic or manual partitioning will be offered.
</p>
<p>Only devices with no mounted partitions and of sufficient size can be
partitioned automatically, these are marked with '+' in the first column.
Any others will be listed with '-' in the first column. They may be
partitioned manually, but great care must be exercised when partitioning
devices which are (partially) in use, and installation to a device which
is actually too small will prove to be difficult, though the manual
option is more flexible than the automatic one.
</p>
<p>The automatically calculated partitioning scheme allows only a limited
amount of tweaking, so if you want more control over the partitioning of
your disk drives and the location of the installation, you will need to
select manual partitioning. The most advanced partition editing option
('Advanced graphical partition manager') uses one of the external tools
gparted or (kde's) partitionmanager, if available.
</p>
<p>The internal manual partitioner also handles selection of the mount
points, so even if you use the advanced option you must complete the
partitioning with the internal partitioner (but not if the automatic
partitioning option is used).
</p>
<p>Selecting one of the devices offered for automatic partitioning will
not immediately cause it to be modified, so try it out without fear. You
can return here and choose manual partitioning later if necessary.
</p>
"""
